@@ -7,14 +7,14 @@
  */
 
 
-#include <stdio.h>
-#include <stdlib.h>
-#include "parser.h"
-#include "xlink.h"
-
 #ifndef __XML_SAX_H__
 #define __XML_SAX_H__
 
+#include <stdio.h>
+#include <stdlib.h>
+#include <libxml/parser.h>
+#include <libxml/xlink.h>
+
 #ifdef __cplusplus
 #define extern "C" {
 #endif
