@@ -12,6 +12,10 @@
 #else
 #include "config.h"
 #endif
+
+#include "xmlversion.h"
+#ifdef LIBXML_DEBUG_ENABLED
+
 #include <stdio.h>
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
@@ -19,12 +23,13 @@
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
-#include "xmlmemory.h"
-#include "tree.h"
-#include "parser.h"
-#include "debugXML.h"
-#include "HTMLtree.h"
-#include "HTMLparser.h"
+#include <libxml/xmlmemory.h>
+#include <libxml/tree.h>
+#include <libxml/parser.h>
+#include <libxml/valid.h>
+#include <libxml/debugXML.h>
+#include <libxml/HTMLtree.h>
+#include <libxml/HTMLparser.h>
 
 #define IS_BLANK(c)							\
   (((c) == '\n') || ((c) == '\r') || ((c) == '\t') || ((c) == ' '))
@@ -38,6 +43,315 @@ void xmlDebugDumpString(FILE *output, const xmlChar *str) {
     fprintf(output, "...");
 }
 
+void xmlDebugDumpDtd(FILE *output, xmlDtdPtr dtd, int depth) {
+    int i;
+    char shift[100];
+
+    for (i = 0;((i < depth) && (i < 25));i++)
+        shift[2 * i] = shift[2 * i + 1] = ' ';
+    shift[2 * i] = shift[2 * i + 1] = 0;
+
+    fprintf(output, shift);
+
+    if (dtd->type != XML_DTD_NODE) {
+	fprintf(output, "PBM: not a DTD\n");
+	return;
+    }
+    if (dtd->name != NULL)
+	fprintf(output, "DTD(%s)", dtd->name);
+    else
+	fprintf(output, "DTD");
+    if (dtd->ExternalID != NULL)
+	fprintf(output, ", PUBLIC %s", dtd->ExternalID);
+    if (dtd->SystemID != NULL)
+	fprintf(output, ", SYSTEM %s", dtd->SystemID);
+    fprintf(output, "\n");
+    /*
+     * Do a bit of checking
+     */
+    if (dtd->parent == NULL)
+	fprintf(output, "PBM: Dtd has no parent\n");
+    if (dtd->doc == NULL)
+	fprintf(output, "PBM: Dtd has no doc\n");
+    if ((dtd->parent != NULL) && (dtd->doc != dtd->parent->doc))
+	fprintf(output, "PBM: Dtd doc differs from parent's one\n");
+    if (dtd->prev == NULL) {
+	if ((dtd->parent != NULL) && (dtd->parent->children != (xmlNodePtr)dtd))
+	    fprintf(output, "PBM: Dtd has no prev and not first of list\n");
+    } else {
+	if (dtd->prev->next != (xmlNodePtr) dtd)
+	    fprintf(output, "PBM: Dtd prev->next : back link wrong\n");
+    }
+    if (dtd->next == NULL) {
+	if ((dtd->parent != NULL) && (dtd->parent->last != (xmlNodePtr) dtd))
+	    fprintf(output, "PBM: Dtd has no next and not last of list\n");
+    } else {
+	if (dtd->next->prev != (xmlNodePtr) dtd)
+	    fprintf(output, "PBM: Dtd next->prev : forward link wrong\n");
+    }
+}
+
+void xmlDebugDumpAttrDecl(FILE *output, xmlAttributePtr attr, int depth) {
+    int i;
+    char shift[100];
+
+    for (i = 0;((i < depth) && (i < 25));i++)
+        shift[2 * i] = shift[2 * i + 1] = ' ';
+    shift[2 * i] = shift[2 * i + 1] = 0;
+
+    fprintf(output, shift);
+
+    if (attr->type != XML_ATTRIBUTE_DECL) {
+	fprintf(output, "PBM: not a Attr\n");
+	return;
+    }
+    if (attr->name != NULL)
+	fprintf(output, "ATTRDECL(%s)", attr->name);
+    else
+	fprintf(output, "PBM ATTRDECL noname!!!");
+    if (attr->elem != NULL)
+	fprintf(output, " for %s", attr->elem);
+    else
+	fprintf(output, " PBM noelem!!!");
+    switch (attr->atype) {
+        case XML_ATTRIBUTE_CDATA:
+	    fprintf(output, " CDATA");
+	    break;
+        case XML_ATTRIBUTE_ID:
+	    fprintf(output, " ID");
+	    break;
+        case XML_ATTRIBUTE_IDREF:
+	    fprintf(output, " IDREF");
+	    break;
+        case XML_ATTRIBUTE_IDREFS:
+	    fprintf(output, " IDREFS");
+	    break;
+        case XML_ATTRIBUTE_ENTITY:
+	    fprintf(output, " ENTITY");
+	    break;
+        case XML_ATTRIBUTE_ENTITIES:
+	    fprintf(output, " ENTITIES");
+	    break;
+        case XML_ATTRIBUTE_NMTOKEN:
+	    fprintf(output, " NMTOKEN");
+	    break;
+        case XML_ATTRIBUTE_NMTOKENS:
+	    fprintf(output, " NMTOKENS");
+	    break;
+        case XML_ATTRIBUTE_ENUMERATION:
+	    fprintf(output, " ENUMERATION");
+	    break;
+        case XML_ATTRIBUTE_NOTATION:
+	    fprintf(output, " NOTATION ");
+	    break;
+    }
+    if (attr->tree != NULL) {
+	int i;
+	xmlEnumerationPtr cur = attr->tree;
+
+	for (i = 0;i < 5; i++) {
+	    if (i != 0)
+		fprintf(output, "|%s", cur->name);
+	    else
+		fprintf(output, " (%s", cur->name);
+	    cur = cur->next;
+	    if (cur == NULL) break;
+	}
+	if (cur == NULL)
+	    fprintf(output, ")");
+	else
+	    fprintf(output, "...)");
+    }
+    switch (attr->def) {
+        case XML_ATTRIBUTE_NONE:
+	    break;
+        case XML_ATTRIBUTE_REQUIRED:
+	    fprintf(output, " REQUIRED");
+	    break;
+        case XML_ATTRIBUTE_IMPLIED:
+	    fprintf(output, " IMPLIED");
+	    break;
+        case XML_ATTRIBUTE_FIXED:
+	    fprintf(output, " FIXED");
+	    break;
+    }
+    if (attr->defaultValue != NULL) {
+	fprintf(output, "\"");
+	xmlDebugDumpString(output, attr->defaultValue);
+	fprintf(output, "\"");
+    }
+    printf("\n");
+
+    /*
+     * Do a bit of checking
+     */
+    if (attr->parent == NULL)
+	fprintf(output, "PBM: Attr has no parent\n");
+    if (attr->doc == NULL)
+	fprintf(output, "PBM: Attr has no doc\n");
+    if ((attr->parent != NULL) && (attr->doc != attr->parent->doc))
+	fprintf(output, "PBM: Attr doc differs from parent's one\n");
+    if (attr->prev == NULL) {
+	if ((attr->parent != NULL) && (attr->parent->children != (xmlNodePtr)attr))
+	    fprintf(output, "PBM: Attr has no prev and not first of list\n");
+    } else {
+	if (attr->prev->next != (xmlNodePtr) attr)
+	    fprintf(output, "PBM: Attr prev->next : back link wrong\n");
+    }
+    if (attr->next == NULL) {
+	if ((attr->parent != NULL) && (attr->parent->last != (xmlNodePtr) attr))
+	    fprintf(output, "PBM: Attr has no next and not last of list\n");
+    } else {
+	if (attr->next->prev != (xmlNodePtr) attr)
+	    fprintf(output, "PBM: Attr next->prev : forward link wrong\n");
+    }
+}
+
+void xmlDebugDumpElemDecl(FILE *output, xmlElementPtr elem, int depth) {
+    int i;
+    char shift[100];
+
+    for (i = 0;((i < depth) && (i < 25));i++)
+        shift[2 * i] = shift[2 * i + 1] = ' ';
+    shift[2 * i] = shift[2 * i + 1] = 0;
+
+    fprintf(output, shift);
+
+    if (elem->type != XML_ELEMENT_DECL) {
+	fprintf(output, "PBM: not a Elem\n");
+	return;
+    }
+    if (elem->name != NULL)
+	fprintf(output, "ELEMDECL(%s)", elem->name);
+    else
+	fprintf(output, "PBM ELEMDECL noname!!!");
+    switch (elem->etype) {
+	case XML_ELEMENT_TYPE_EMPTY: 
+	    fprintf(output, ", EMPTY");
+	    break;
+	case XML_ELEMENT_TYPE_ANY: 
+	    fprintf(output, ", ANY");
+	    break;
+	case XML_ELEMENT_TYPE_MIXED: 
+	    fprintf(output, ", MIXED ");
+	    break;
+	case XML_ELEMENT_TYPE_ELEMENT: 
+	    fprintf(output, ", MIXED ");
+	    break;
+    }
+    if (elem->content != NULL) {
+	char buf[1001];
+
+	buf[0] = 0;
+	xmlSprintfElementContent(buf, elem->content, 1);
+	buf[1000] = 0;
+	fprintf(output, "%s", buf);
+    }
+    printf("\n");
+
+    /*
+     * Do a bit of checking
+     */
+    if (elem->parent == NULL)
+	fprintf(output, "PBM: Elem has no parent\n");
+    if (elem->doc == NULL)
+	fprintf(output, "PBM: Elem has no doc\n");
+    if ((elem->parent != NULL) && (elem->doc != elem->parent->doc))
+	fprintf(output, "PBM: Elem doc differs from parent's one\n");
+    if (elem->prev == NULL) {
+	if ((elem->parent != NULL) && (elem->parent->children != (xmlNodePtr)elem))
+	    fprintf(output, "PBM: Elem has no prev and not first of list\n");
+    } else {
+	if (elem->prev->next != (xmlNodePtr) elem)
+	    fprintf(output, "PBM: Elem prev->next : back link wrong\n");
+    }
+    if (elem->next == NULL) {
+	if ((elem->parent != NULL) && (elem->parent->last != (xmlNodePtr) elem))
+	    fprintf(output, "PBM: Elem has no next and not last of list\n");
+    } else {
+	if (elem->next->prev != (xmlNodePtr) elem)
+	    fprintf(output, "PBM: Elem next->prev : forward link wrong\n");
+    }
+}
+
+void xmlDebugDumpEntityDecl(FILE *output, xmlEntityPtr ent, int depth) {
+    int i;
+    char shift[100];
+
+    for (i = 0;((i < depth) && (i < 25));i++)
+        shift[2 * i] = shift[2 * i + 1] = ' ';
+    shift[2 * i] = shift[2 * i + 1] = 0;
+
+    fprintf(output, shift);
+
+    if (ent->type != XML_ENTITY_DECL) {
+	fprintf(output, "PBM: not a Entity decl\n");
+	return;
+    }
+    if (ent->name != NULL)
+	fprintf(output, "ENTITYDECL(%s)", ent->name);
+    else
+	fprintf(output, "PBM ENTITYDECL noname!!!");
+    switch (ent->etype) {
+	case XML_INTERNAL_GENERAL_ENTITY: 
+	    fprintf(output, ", internal\n");
+	    break;
+	case XML_EXTERNAL_GENERAL_PARSED_ENTITY: 
+	    fprintf(output, ", external parsed\n");
+	    break;
+	case XML_EXTERNAL_GENERAL_UNPARSED_ENTITY: 
+	    fprintf(output, ", unparsed\n");
+	    break;
+	case XML_INTERNAL_PARAMETER_ENTITY: 
+	    fprintf(output, ", parameter\n");
+	    break;
+	case XML_EXTERNAL_PARAMETER_ENTITY: 
+	    fprintf(output, ", external parameter\n");
+	    break;
+	case XML_INTERNAL_PREDEFINED_ENTITY: 
+	    fprintf(output, ", predefined\n");
+	    break;
+    }
+    if (ent->ExternalID) {
+        fprintf(output, shift);
+        fprintf(output, "ExternalID=%s\n", ent->ExternalID);
+    }
+    if (ent->SystemID) {
+        fprintf(output, shift);
+        fprintf(output, "SystemID=%s\n", ent->SystemID);
+    }
+    if (ent->content) {
+        fprintf(output, shift);
+	fprintf(output, "content=");
+	xmlDebugDumpString(output, ent->content);
+	fprintf(output, "\n");
+    }
+
+    /*
+     * Do a bit of checking
+     */
+    if (ent->parent == NULL)
+	fprintf(output, "PBM: Ent has no parent\n");
+    if (ent->doc == NULL)
+	fprintf(output, "PBM: Ent has no doc\n");
+    if ((ent->parent != NULL) && (ent->doc != ent->parent->doc))
+	fprintf(output, "PBM: Ent doc differs from parent's one\n");
+    if (ent->prev == NULL) {
+	if ((ent->parent != NULL) && (ent->parent->children != (xmlNodePtr)ent))
+	    fprintf(output, "PBM: Ent has no prev and not first of list\n");
+    } else {
+	if (ent->prev->next != (xmlNodePtr) ent)
+	    fprintf(output, "PBM: Ent prev->next : back link wrong\n");
+    }
+    if (ent->next == NULL) {
+	if ((ent->parent != NULL) && (ent->parent->last != (xmlNodePtr) ent))
+	    fprintf(output, "PBM: Ent has no next and not last of list\n");
+    } else {
+	if (ent->next->prev != (xmlNodePtr) ent)
+	    fprintf(output, "PBM: Ent next->prev : forward link wrong\n");
+    }
+}
+
 void xmlDebugDumpNamespace(FILE *output, xmlNsPtr ns, int depth) {
     int i;
     char shift[100];
@@ -74,7 +388,7 @@ void xmlDebugDumpEntity(FILE *output, xmlEntityPtr ent, int depth) {
     shift[2 * i] = shift[2 * i + 1] = 0;
 
     fprintf(output, shift);
-    switch (ent->type) {
+    switch (ent->etype) {
         case XML_INTERNAL_GENERAL_ENTITY:
 	    fprintf(output, "INTERNAL_GENERAL_ENTITY ");
 	    break;
@@ -91,7 +405,7 @@ void xmlDebugDumpEntity(FILE *output, xmlEntityPtr ent, int depth) {
 	    fprintf(output, "EXTERNAL_PARAMETER_ENTITY ");
 	    break;
 	default:
-	    fprintf(output, "ENTITY_%d ! ", ent->type);
+	    fprintf(output, "ENTITY_%d ! ", ent->etype);
     }
     fprintf(output, "%s\n", ent->name);
     if (ent->ExternalID) {
@@ -119,9 +433,31 @@ void xmlDebugDumpAttr(FILE *output, xmlAttrPtr attr, int depth) {
     shift[2 * i] = shift[2 * i + 1] = 0;
 
     fprintf(output, shift);
+
     fprintf(output, "ATTRIBUTE %s\n", attr->name);
-    if (attr->val != NULL) 
-        xmlDebugDumpNodeList(output, attr->val, depth + 1);
+    if (attr->children != NULL) 
+        xmlDebugDumpNodeList(output, attr->children, depth + 1);
+
+    /*
+     * Do a bit of checking
+     */
+    if (attr->parent == NULL)
+	fprintf(output, "PBM: Attr has no parent\n");
+    if (attr->doc == NULL)
+	fprintf(output, "PBM: Attr has no doc\n");
+    if ((attr->parent != NULL) && (attr->doc != attr->parent->doc))
+	fprintf(output, "PBM: Attr doc differs from parent's one\n");
+    if (attr->prev == NULL) {
+	if ((attr->parent != NULL) && (attr->parent->properties != attr))
+	    fprintf(output, "PBM: Attr has no prev and not first of list\n");
+    } else {
+	if (attr->prev->next != attr)
+	    fprintf(output, "PBM: Attr prev->next : back link wrong\n");
+    }
+    if (attr->next != NULL) {
+	if (attr->next->prev != attr)
+	    fprintf(output, "PBM: Attr next->prev : forward link wrong\n");
+    }
 }
 
 void xmlDebugDumpAttrList(FILE *output, xmlAttrPtr attr, int depth) {
@@ -139,9 +475,9 @@ void xmlDebugDumpOneNode(FILE *output, xmlNodePtr node, int depth) {
         shift[2 * i] = shift[2 * i + 1] = ' ';
     shift[2 * i] = shift[2 * i + 1] = 0;
 
-    fprintf(output, shift);
     switch (node->type) {
 	case XML_ELEMENT_NODE:
+	    fprintf(output, shift);
 	    fprintf(output, "ELEMENT ");
 	    if (node->ns != NULL)
 	        fprintf(output, "%s:%s\n", node->ns->prefix, node->name);
@@ -149,40 +485,63 @@ void xmlDebugDumpOneNode(FILE *output, xmlNodePtr node, int depth) {
 	        fprintf(output, "%s\n", node->name);
 	    break;
 	case XML_ATTRIBUTE_NODE:
+	    fprintf(output, shift);
 	    fprintf(output, "Error, ATTRIBUTE found here\n");
 	    break;
 	case XML_TEXT_NODE:
+	    fprintf(output, shift);
 	    fprintf(output, "TEXT\n");
 	    break;
 	case XML_CDATA_SECTION_NODE:
+	    fprintf(output, shift);
 	    fprintf(output, "CDATA_SECTION\n");
 	    break;
 	case XML_ENTITY_REF_NODE:
-	    fprintf(output, "ENTITY_REF\n");
+	    fprintf(output, shift);
+	    fprintf(output, "ENTITY_REF(%s)\n", node->name);
 	    break;
 	case XML_ENTITY_NODE:
+	    fprintf(output, shift);
 	    fprintf(output, "ENTITY\n");
 	    break;
 	case XML_PI_NODE:
+	    fprintf(output, shift);
 	    fprintf(output, "PI %s\n", node->name);
 	    break;
 	case XML_COMMENT_NODE:
+	    fprintf(output, shift);
 	    fprintf(output, "COMMENT\n");
 	    break;
 	case XML_DOCUMENT_NODE:
 	case XML_HTML_DOCUMENT_NODE:
+	    fprintf(output, shift);
 	    fprintf(output, "Error, DOCUMENT found here\n");
 	    break;
 	case XML_DOCUMENT_TYPE_NODE:
+	    fprintf(output, shift);
 	    fprintf(output, "DOCUMENT_TYPE\n");
 	    break;
 	case XML_DOCUMENT_FRAG_NODE:
+	    fprintf(output, shift);
 	    fprintf(output, "DOCUMENT_FRAG\n");
 	    break;
 	case XML_NOTATION_NODE:
 	    fprintf(output, "NOTATION\n");
 	    break;
+	case XML_DTD_NODE:
+	    xmlDebugDumpDtd(output, (xmlDtdPtr) node, depth);
+	    return;
+	case XML_ELEMENT_DECL:
+	    xmlDebugDumpElemDecl(output, (xmlElementPtr) node, depth);
+	    return;
+	case XML_ATTRIBUTE_DECL:
+	    xmlDebugDumpAttrDecl(output, (xmlAttributePtr) node, depth);
+	    return;
+        case XML_ENTITY_DECL:
+	    xmlDebugDumpEntityDecl(output, (xmlEntityPtr) node, depth);
+	    return;
 	default:
+	    fprintf(output, shift);
 	    fprintf(output, "NODE_%d\n", node->type);
     }
     if (node->doc == NULL) {
@@ -210,12 +569,35 @@ void xmlDebugDumpOneNode(FILE *output, xmlNodePtr node, int depth) {
 	if (ent != NULL)
 	    xmlDebugDumpEntity(output, ent, depth + 1);
     }
+    /*
+     * Do a bit of checking
+     */
+    if (node->parent == NULL)
+	fprintf(output, "PBM: Node has no parent\n");
+    if (node->doc == NULL)
+	fprintf(output, "PBM: Node has no doc\n");
+    if ((node->parent != NULL) && (node->doc != node->parent->doc))
+	fprintf(output, "PBM: Node doc differs from parent's one\n");
+    if (node->prev == NULL) {
+	if ((node->parent != NULL) && (node->parent->children != node))
+	    fprintf(output, "PBM: Node has no prev and not first of list\n");
+    } else {
+	if (node->prev->next != node)
+	    fprintf(output, "PBM: Node prev->next : back link wrong\n");
+    }
+    if (node->next == NULL) {
+	if ((node->parent != NULL) && (node->parent->last != node))
+	    fprintf(output, "PBM: Node has no next and not last of list\n");
+    } else {
+	if (node->next->prev != node)
+	    fprintf(output, "PBM: Node next->prev : forward link wrong\n");
+    }
 }
 
 void xmlDebugDumpNode(FILE *output, xmlNodePtr node, int depth) {
     xmlDebugDumpOneNode(output, node, depth);
-    if (node->childs != NULL)
-	xmlDebugDumpNodeList(output, node->childs, depth + 1);
+    if (node->children != NULL)
+	xmlDebugDumpNodeList(output, node->children, depth + 1);
 }
 
 void xmlDebugDumpNodeList(FILE *output, xmlNodePtr node, int depth) {
@@ -306,8 +688,8 @@ void xmlDebugDumpDocument(FILE *output, xmlDocPtr doc) {
     xmlDebugDumpDocumentHead(output, doc);
     if (((doc->type == XML_DOCUMENT_NODE) ||
          (doc->type == XML_HTML_DOCUMENT_NODE)) &&
-        (doc->root != NULL))
-        xmlDebugDumpNodeList(output, doc->root, 1);
+        (doc->children != NULL))
+        xmlDebugDumpNodeList(output, doc->children, 1);
 }    
 
 void xmlDebugDumpEntities(FILE *output, xmlDocPtr doc) {
@@ -368,27 +750,27 @@ void xmlDebugDumpEntities(FILE *output, xmlDocPtr doc) {
 	                            doc->intSubset->entities;
 	fprintf(output, "Entities in internal subset\n");
 	for (i = 0;i < table->nb_entities;i++) {
-	    cur = &table->table[i];
+	    cur = table->table[i];
 	    fprintf(output, "%d : %s : ", i, cur->name);
-	    switch (cur->type) {
+	    switch (cur->etype) {
 		case XML_INTERNAL_GENERAL_ENTITY:
-		    fprintf(output, "INTERNAL GENERAL");
+		    fprintf(output, "INTERNAL GENERAL, ");
 		    break;
 		case XML_EXTERNAL_GENERAL_PARSED_ENTITY:
-		    fprintf(output, "EXTERNAL PARSED");
+		    fprintf(output, "EXTERNAL PARSED, ");
 		    break;
 		case XML_EXTERNAL_GENERAL_UNPARSED_ENTITY:
-		    fprintf(output, "EXTERNAL UNPARSED");
+		    fprintf(output, "EXTERNAL UNPARSED, ");
 		    break;
 		case XML_INTERNAL_PARAMETER_ENTITY:
-		    fprintf(output, "INTERNAL PARAMETER");
+		    fprintf(output, "INTERNAL PARAMETER, ");
 		    break;
 		case XML_EXTERNAL_PARAMETER_ENTITY:
-		    fprintf(output, "EXTERNAL PARAMETER");
+		    fprintf(output, "EXTERNAL PARAMETER, ");
 		    break;
 		default:
 		    fprintf(output, "UNKNOWN TYPE %d",
-			    cur->type);
+			    cur->etype);
 	    }
 	    if (cur->ExternalID != NULL) 
 	        fprintf(output, "ID \"%s\"", cur->ExternalID);
@@ -407,27 +789,27 @@ void xmlDebugDumpEntities(FILE *output, xmlDocPtr doc) {
 	                            doc->extSubset->entities;
 	fprintf(output, "Entities in external subset\n");
 	for (i = 0;i < table->nb_entities;i++) {
-	    cur = &table->table[i];
+	    cur = table->table[i];
 	    fprintf(output, "%d : %s : ", i, cur->name);
-	    switch (cur->type) {
+	    switch (cur->etype) {
 		case XML_INTERNAL_GENERAL_ENTITY:
-		    fprintf(output, "INTERNAL GENERAL");
+		    fprintf(output, "INTERNAL GENERAL, ");
 		    break;
 		case XML_EXTERNAL_GENERAL_PARSED_ENTITY:
-		    fprintf(output, "EXTERNAL PARSED");
+		    fprintf(output, "EXTERNAL PARSED, ");
 		    break;
 		case XML_EXTERNAL_GENERAL_UNPARSED_ENTITY:
-		    fprintf(output, "EXTERNAL UNPARSED");
+		    fprintf(output, "EXTERNAL UNPARSED, ");
 		    break;
 		case XML_INTERNAL_PARAMETER_ENTITY:
-		    fprintf(output, "INTERNAL PARAMETER");
+		    fprintf(output, "INTERNAL PARAMETER, ");
 		    break;
 		case XML_EXTERNAL_PARAMETER_ENTITY:
-		    fprintf(output, "EXTERNAL PARAMETER");
+		    fprintf(output, "EXTERNAL PARAMETER, ");
 		    break;
 		default:
 		    fprintf(output, "UNKNOWN TYPE %d",
-			    cur->type);
+			    cur->etype);
 	    }
 	    if (cur->ExternalID != NULL) 
 	        fprintf(output, "ID \"%s\"", cur->ExternalID);
@@ -449,14 +831,14 @@ static int xmlLsCountNode(xmlNodePtr node) {
 
     switch (node->type) {
 	case XML_ELEMENT_NODE:
-	    list = node->childs;
+	    list = node->children;
 	    break;
 	case XML_DOCUMENT_NODE:
 	case XML_HTML_DOCUMENT_NODE:
-	    list = ((xmlDocPtr) node)->root;
+	    list = ((xmlDocPtr) node)->children;
 	    break;
 	case XML_ATTRIBUTE_NODE:
-	    list = ((xmlAttrPtr) node)->val;
+	    list = ((xmlAttrPtr) node)->children;
 	    break;
 	case XML_TEXT_NODE:
 	case XML_CDATA_SECTION_NODE:
@@ -475,6 +857,10 @@ static int xmlLsCountNode(xmlNodePtr node) {
 	case XML_ENTITY_NODE:
 	case XML_DOCUMENT_FRAG_NODE:
 	case XML_NOTATION_NODE:
+	case XML_DTD_NODE:
+        case XML_ELEMENT_DECL:
+        case XML_ATTRIBUTE_DECL:
+        case XML_ENTITY_DECL:
 	    ret = 1;
 	    break;
     }
@@ -621,9 +1007,9 @@ xmlShellList(xmlShellCtxtPtr ctxt, char *arg, xmlNodePtr node,
 
     if ((node->type == XML_DOCUMENT_NODE) ||
         (node->type == XML_HTML_DOCUMENT_NODE)) {
-        cur = ((xmlDocPtr) node)->root;
-    } else if (node->childs != NULL) {
-        cur = node->childs;
+        cur = ((xmlDocPtr) node)->children;
+    } else if (node->children != NULL) {
+        cur = node->children;
     } else {
 	xmlLsOneNode(stdout, node);
         return(0);
@@ -677,10 +1063,17 @@ int
 xmlShellCat(xmlShellCtxtPtr ctxt, char *arg, xmlNodePtr node,
                   xmlNodePtr node2) {
     if (ctxt->doc->type == XML_HTML_DOCUMENT_NODE) {
+#ifdef LIBXML_HTML_ENABLED
 	if (node->type == XML_HTML_DOCUMENT_NODE)
 	    htmlDocDump(stdout, (htmlDocPtr) node);
 	else
 	    htmlNodeDumpFile(stdout, ctxt->doc, node);
+#else
+	if (node->type == XML_DOCUMENT_NODE)
+	    xmlDocDump(stdout, (xmlDocPtr) node);
+	else
+	    xmlElemDump(stdout, ctxt->doc, node);
+#endif /* LIBXML_HTML_ENABLED */
     } else {
 	if (node->type == XML_DOCUMENT_NODE)
 	    xmlDocDump(stdout, (xmlDocPtr) node);
@@ -713,7 +1106,12 @@ xmlShellLoad(xmlShellCtxtPtr ctxt, char *filename, xmlNodePtr node,
 	html = (ctxt->doc->type == XML_HTML_DOCUMENT_NODE);
 
     if (html) {
+#ifdef LIBXML_HTML_ENABLED
 	doc = htmlParseFile(filename, NULL);
+#else	
+	printf("HTML support not compiled in\n");
+	doc = NULL;
+#endif /* LIBXML_HTML_ENABLED */
     } else {
 	doc = xmlParseFile(filename);
     }
@@ -722,11 +1120,15 @@ xmlShellLoad(xmlShellCtxtPtr ctxt, char *filename, xmlNodePtr node,
 	    xmlFreeDoc(ctxt->doc);
 	}
 	ctxt->loaded = 1;
+#ifdef LIBXML_XPATH_ENABLED
 	xmlXPathFreeContext(ctxt->pctxt);
+#endif /* LIBXML_XPATH_ENABLED */
 	xmlFree(ctxt->filename);
 	ctxt->doc = doc;
 	ctxt->node = (xmlNodePtr) doc;	 
+#ifdef LIBXML_XPATH_ENABLED
 	ctxt->pctxt = xmlXPathNewContext(doc);
+#endif /* LIBXML_XPATH_ENABLED */
 	ctxt->filename = (char *) xmlStrdup((xmlChar *) filename);
     } else
         return(-1);
@@ -769,10 +1171,17 @@ xmlShellWrite(xmlShellCtxtPtr ctxt, char *filename, xmlNodePtr node,
 	    }
 	    break;
         case XML_HTML_DOCUMENT_NODE:
+#ifdef LIBXML_HTML_ENABLED
 	    if (htmlSaveFile((char *) filename, ctxt->doc) < 0) {
 		fprintf(stderr, "Failed to write to %s\n", filename);
 		return(-1);
 	    }
+#else
+	    if (xmlSaveFile((char *) filename, ctxt->doc) < -1) {
+		fprintf(stderr, "Failed to write to %s\n", filename);
+		return(-1);
+	    }
+#endif /* LIBXML_HTML_ENABLED */
 	    break;
 	default: {
 	    FILE *f;
@@ -821,9 +1230,15 @@ xmlShellSave(xmlShellCtxtPtr ctxt, char *filename, xmlNodePtr node,
 	    }
 	    break;
         case XML_HTML_DOCUMENT_NODE:
+#ifdef LIBXML_HTML_ENABLED
 	    if (htmlSaveFile((char *) filename, ctxt->doc) < 0) {
 		fprintf(stderr, "Failed to save to %s\n", filename);
 	    }
+#else
+	    if (xmlSaveFile((char *) filename, ctxt->doc) < 0) {
+		fprintf(stderr, "Failed to save to %s\n", filename);
+	    }
+#endif /* LIBXML_HTML_ENABLED */
 	    break;
 	default:
 	    fprintf(stderr, 
@@ -910,10 +1325,10 @@ xmlShellDu(xmlShellCtxtPtr ctxt, char *arg, xmlNodePtr tree,
 
         if ((node->type == XML_DOCUMENT_NODE) ||
             (node->type == XML_HTML_DOCUMENT_NODE)) {
-	    node = ((xmlDocPtr) node)->root;
-        } else if (node->childs != NULL) {
+	    node = ((xmlDocPtr) node)->children;
+        } else if (node->children != NULL) {
 	    /* deep first */
-	    node = node->childs;
+	    node = node->children;
 	    indent++;
 	} else if ((node != tree) && (node->next != NULL)) {
 	    /* then siblings */
@@ -1008,7 +1423,7 @@ xmlShellPwd(xmlShellCtxtPtr ctxt, char *buffer, xmlNodePtr node,
 	} else if (cur->type == XML_ATTRIBUTE_NODE) {
 	    sep = '@';
 	    name = (const char *) (((xmlAttrPtr) cur)->name);
-	    next = ((xmlAttrPtr) cur)->node;
+	    next = ((xmlAttrPtr) cur)->parent;
 	} else {
 	    next = cur->parent;
 	}
@@ -1062,11 +1477,13 @@ xmlShell(xmlDocPtr doc, char *filename, xmlShellReadlineFunc input,
     ctxt->filename = (char *) xmlStrdup((xmlChar *) filename);
     ctxt->node = (xmlNodePtr) ctxt->doc;	 
 
+#ifdef LIBXML_XPATH_ENABLED
     ctxt->pctxt = xmlXPathNewContext(ctxt->doc);
     if (ctxt->pctxt == NULL) {
 	xmlFree(ctxt);
 	return;
     }
+#endif /* LIBXML_XPATH_ENABLED */
     while (1) {
         if (ctxt->node == (xmlNodePtr) ctxt->doc)
 	    sprintf(prompt, "%s > ", "/");
@@ -1121,10 +1538,14 @@ xmlShell(xmlDocPtr doc, char *filename, xmlShellReadlineFunc input,
 		    xmlShellList(ctxt, NULL, ctxt->node, NULL);
 	    } else {
 	        ctxt->pctxt->node = ctxt->node;
+#ifdef LIBXML_XPATH_ENABLED
 		if (ctxt->pctxt->nodelist != NULL)
 		    xmlXPathFreeNodeSet(ctxt->pctxt->nodelist);
 	        ctxt->pctxt->nodelist = xmlXPathNodeSetCreate(ctxt->node);
 		list = xmlXPathEval((xmlChar *) arg, ctxt->pctxt);
+#else
+		list = NULL;
+#endif /* LIBXML_XPATH_ENABLED */
 		if (list != NULL) {
 		    switch (list->type) {
 			case XPATH_UNDEFINED:
@@ -1157,8 +1578,10 @@ xmlShell(xmlDocPtr doc, char *filename, xmlShellReadlineFunc input,
 		} else {
 		    fprintf(stderr, "%s: no such node\n", arg);
 		}
+#ifdef LIBXML_XPATH_ENABLED
 		if (ctxt->pctxt->nodelist != NULL)
 		    xmlXPathFreeNodeSet(ctxt->pctxt->nodelist);
+#endif /* LIBXML_XPATH_ENABLED */
 		ctxt->pctxt->nodelist = NULL;
 	    }
 	} else if (!strcmp(command, "cd")) {
@@ -1166,10 +1589,14 @@ xmlShell(xmlDocPtr doc, char *filename, xmlShellReadlineFunc input,
 		ctxt->node = (xmlNodePtr) ctxt->doc;
 	    } else {
 	        ctxt->pctxt->node = ctxt->node;
+#ifdef LIBXML_XPATH_ENABLED
 		if (ctxt->pctxt->nodelist != NULL)
 		    xmlXPathFreeNodeSet(ctxt->pctxt->nodelist);
 	        ctxt->pctxt->nodelist = xmlXPathNodeSetCreate(ctxt->node);
 		list = xmlXPathEval((xmlChar *) arg, ctxt->pctxt);
+#else
+		list = NULL;
+#endif /* LIBXML_XPATH_ENABLED */
 		if (list != NULL) {
 		    switch (list->type) {
 			case XPATH_UNDEFINED:
@@ -1196,8 +1623,10 @@ xmlShell(xmlDocPtr doc, char *filename, xmlShellReadlineFunc input,
 		} else {
 		    fprintf(stderr, "%s: no such node\n", arg);
 		}
+#ifdef LIBXML_XPATH_ENABLED
 		if (ctxt->pctxt->nodelist != NULL)
 		    xmlXPathFreeNodeSet(ctxt->pctxt->nodelist);
+#endif /* LIBXML_XPATH_ENABLED */
 		ctxt->pctxt->nodelist = NULL;
 	    }
 	} else if (!strcmp(command, "cat")) {
@@ -1205,10 +1634,14 @@ xmlShell(xmlDocPtr doc, char *filename, xmlShellReadlineFunc input,
 		xmlShellCat(ctxt, NULL, ctxt->node, NULL);
 	    } else {
 	        ctxt->pctxt->node = ctxt->node;
+#ifdef LIBXML_XPATH_ENABLED
 		if (ctxt->pctxt->nodelist != NULL)
 		    xmlXPathFreeNodeSet(ctxt->pctxt->nodelist);
 	        ctxt->pctxt->nodelist = xmlXPathNodeSetCreate(ctxt->node);
 		list = xmlXPathEval((xmlChar *) arg, ctxt->pctxt);
+#else
+		list = NULL;
+#endif /* LIBXML_XPATH_ENABLED */
 		if (list != NULL) {
 		    switch (list->type) {
 			case XPATH_UNDEFINED:
@@ -1238,8 +1671,10 @@ xmlShell(xmlDocPtr doc, char *filename, xmlShellReadlineFunc input,
 		} else {
 		    fprintf(stderr, "%s: no such node\n", arg);
 		}
+#ifdef LIBXML_XPATH_ENABLED
 		if (ctxt->pctxt->nodelist != NULL)
 		    xmlXPathFreeNodeSet(ctxt->pctxt->nodelist);
+#endif /* LIBXML_XPATH_ENABLED */
 		ctxt->pctxt->nodelist = NULL;
 	    }
 	} else {
@@ -1247,7 +1682,9 @@ xmlShell(xmlDocPtr doc, char *filename, xmlShellReadlineFunc input,
 	}
 	free(cmdline); /* not xmlFree here ! */
     }
+#ifdef LIBXML_XPATH_ENABLED
     xmlXPathFreeContext(ctxt->pctxt);
+#endif /* LIBXML_XPATH_ENABLED */
     if (ctxt->loaded) {
         xmlFreeDoc(ctxt->doc);
     }
@@ -1256,3 +1693,4 @@ xmlShell(xmlDocPtr doc, char *filename, xmlShellReadlineFunc input,
         free(cmdline); /* not xmlFree here ! */
 }
 
+#endif /* LIBXML_DEBUG_ENABLED */
