@@ -1,5 +1,6 @@
-/*
- * nanoftp.c: basic FTP client support
+/**
+ * ftp.c: basic handling of an FTP command connection to check for
+ *        directory availability. No transfer is needed.
  *
  *  Reference: RFC 959
  */
@@ -10,16 +11,23 @@
 #else
 #include "config.h"
 #endif
+#include "xmlversion.h"
 
+#ifdef LIBXML_FTP_ENABLED
 #include <stdio.h>
 #include <string.h>
 
-#ifdef HAVE_STDLIB_H
-#include <stdlib.h>
+#ifdef HAVE_CTYPE_H
+#include <ctype.h>
 #endif
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
+
+#include <sys/types.h>
+#ifdef HAVE_SYS_TIME_H
+#include <sys/time.h>
+#endif
 #ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
 #endif
@@ -44,12 +52,18 @@
 #ifdef HAVE_SYS_SELECT_H
 #include <sys/select.h>
 #endif
+#ifdef HAVE_RESOLV_H
+#include <resolv.h>
+#endif
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
 #ifdef HAVE_STRINGS_H
 #include <strings.h>
 #endif
 
-#include "xmlmemory.h"
-#include "nanoftp.h"
+#include <libxml/xmlmemory.h>
+#include <libxml/nanoftp.h>
 
 /* #define DEBUG_FTP 1  */
 #ifdef STANDALONE
@@ -63,9 +77,6 @@ static char hostname[100];
 #define FTP_COMMAND_OK		200
 #define FTP_SYNTAX_ERROR	500
 #define FTP_GET_PASSWD		331
-#define FTP_BUF_SIZE		512
-
-#define XML_NANO_MAX_URLBUF	4096
 
 typedef struct xmlNanoFTPCtxt {
     char *protocol;	/* the protocol name */
@@ -80,11 +91,6 @@ typedef struct xmlNanoFTPCtxt {
     int dataFd;		/* the file descriptor for the data socket */
     int state;		/* WRITE / READ / CLOSED */
     int returnValue;	/* the protocol return value */
-    /* buffer for data received from the control connection */
-    char controlBuf[FTP_BUF_SIZE + 1];
-    int controlBufIndex;
-    int controlBufUsed;
-    int controlBufAnswer;
 } xmlNanoFTPCtxt, *xmlNanoFTPCtxtPtr;
 
 static int initialized = 0;
@@ -205,7 +211,7 @@ static void
 xmlNanoFTPScanURL(void *ctx, const char *URL) {
     xmlNanoFTPCtxtPtr ctxt = (xmlNanoFTPCtxtPtr) ctx;
     const char *cur = URL;
-    char buf[XML_NANO_MAX_URLBUF];
+    char buf[4096];
     int index = 0;
     int port = 0;
 
@@ -223,7 +229,7 @@ xmlNanoFTPScanURL(void *ctx, const char *URL) {
     }
     if (URL == NULL) return;
     buf[index] = 0;
-    while ((*cur != 0) && (index < XML_NANO_MAX_URLBUF - 1)) {
+    while (*cur != 0) {
         if ((cur[0] == ':') && (cur[1] == '/') && (cur[2] == '/')) {
 	    buf[index] = 0;
 	    ctxt->protocol = xmlMemStrdup(buf);
@@ -236,7 +242,7 @@ xmlNanoFTPScanURL(void *ctx, const char *URL) {
     if (*cur == 0) return;
 
     buf[index] = 0;
-    while (index < XML_NANO_MAX_URLBUF - 1) {
+    while (1) {
         if (cur[0] == ':') {
 	    buf[index] = 0;
 	    ctxt->hostname = xmlMemStrdup(buf);
@@ -265,7 +271,7 @@ xmlNanoFTPScanURL(void *ctx, const char *URL) {
     else {
         index = 0;
         buf[index] = 0;
-	while ((*cur != 0) && (index < XML_NANO_MAX_URLBUF-1))
+	while (*cur != 0)
 	    buf[index++] = *cur++;
 	buf[index] = 0;
 	ctxt->path = xmlMemStrdup(buf);
@@ -290,7 +296,7 @@ int
 xmlNanoFTPUpdateURL(void *ctx, const char *URL) {
     xmlNanoFTPCtxtPtr ctxt = (xmlNanoFTPCtxtPtr) ctx;
     const char *cur = URL;
-    char buf[XML_NANO_MAX_URLBUF];
+    char buf[4096];
     int index = 0;
     int port = 0;
 
@@ -303,7 +309,7 @@ xmlNanoFTPUpdateURL(void *ctx, const char *URL) {
     if (ctxt->hostname == NULL)
 	return(-1);
     buf[index] = 0;
-    while ((*cur != 0) && (index < XML_NANO_MAX_URLBUF-1)) {
+    while (*cur != 0) {
         if ((cur[0] == ':') && (cur[1] == '/') && (cur[2] == '/')) {
 	    buf[index] = 0;
 	    if (strcmp(ctxt->protocol, buf))
@@ -355,7 +361,7 @@ xmlNanoFTPUpdateURL(void *ctx, const char *URL) {
     else {
         index = 0;
         buf[index] = 0;
-	while ((*cur != 0) && (index < XML_NANO_MAX_URLBUF-1))
+	while (*cur != 0)
 	    buf[index++] = *cur++;
 	buf[index] = 0;
 	ctxt->path = xmlMemStrdup(buf);
@@ -376,7 +382,7 @@ xmlNanoFTPUpdateURL(void *ctx, const char *URL) {
 void
 xmlNanoFTPScanProxy(const char *URL) {
     const char *cur = URL;
-    char buf[XML_NANO_MAX_URLBUF];
+    char buf[4096];
     int index = 0;
     int port = 0;
 
@@ -395,7 +401,7 @@ xmlNanoFTPScanProxy(const char *URL) {
 #endif
     if (URL == NULL) return;
     buf[index] = 0;
-    while ((*cur != 0) && (index < XML_NANO_MAX_URLBUF-1)) {
+    while (*cur != 0) {
         if ((cur[0] == ':') && (cur[1] == '/') && (cur[2] == '/')) {
 	    buf[index] = 0;
 	    index = 0;
@@ -442,7 +448,7 @@ xmlNanoFTPScanProxy(const char *URL) {
  * Returns an FTP context or NULL in case of error.
  */
 
-void*
+void *
 xmlNanoFTPNewCtxt(const char *URL) {
     xmlNanoFTPCtxtPtr ret;
 
@@ -453,8 +459,6 @@ xmlNanoFTPNewCtxt(const char *URL) {
     ret->port = 21;
     ret->passive = 1;
     ret->returnValue = 0;
-    ret->controlBufIndex = 0;
-    ret->controlBufUsed = 0;
 
     if (URL != NULL)
 	xmlNanoFTPScanURL(ret, URL);
@@ -479,20 +483,12 @@ xmlNanoFTPFreeCtxt(void * ctx) {
     ctxt->passive = 1;
     if (ctxt->controlFd >= 0) close(ctxt->controlFd);
     ctxt->controlFd = -1;
-    ctxt->controlBufIndex = -1;
-    ctxt->controlBufUsed = -1;
     xmlFree(ctxt);
 }
 
 /**
- * xmlNanoFTPParseResponse:
- * @ctx:  the FTP connection context
- * @buf:  the buffer containing the response
- * @len:  the buffer length
- * 
  * Parsing of the server answer, we just extract the code.
- *
- * returns 0 for errors
+ * return 0 for errors
  *     +XXX for last line of response
  *     -XXX for response to be continued
  */
@@ -521,137 +517,60 @@ xmlNanoFTPParseResponse(void *ctx, char *buf, int len) {
     return(val);
 }
 
-/**
- * xmlNanoFTPGetMore:
- * @ctx:  an FTP context
- *
- * Read more information from the FTP control connection
- * Returns the number of bytes read, < 0 indicates an error
- */
-static int
-xmlNanoFTPGetMore(void *ctx) {
-    xmlNanoFTPCtxtPtr ctxt = (xmlNanoFTPCtxtPtr) ctx;
-    int len;
-    int size;
-
-    if ((ctxt->controlBufIndex < 0) || (ctxt->controlBufIndex > FTP_BUF_SIZE)) {
-#ifdef DEBUG_FTP
-        printf("xmlNanoFTPGetMore : controlBufIndex = %d\n",
-		ctxt->controlBufIndex);
-#endif
-	return(-1);
-    }
-
-    if ((ctxt->controlBufUsed < 0) || (ctxt->controlBufUsed > FTP_BUF_SIZE)) {
-#ifdef DEBUG_FTP
-        printf("xmlNanoFTPGetMore : controlBufUsed = %d\n",
-		ctxt->controlBufUsed);
-#endif
-	return(-1);
-    }
-    if (ctxt->controlBufIndex > ctxt->controlBufUsed) {
-#ifdef DEBUG_FTP
-        printf("xmlNanoFTPGetMore : controlBufIndex > controlBufUsed %d > %d\n",
-	       ctxt->controlBufIndex, ctxt->controlBufUsed);
-#endif
-	return(-1);
-    }
-
-    /*
-     * First pack the control buffer
-     */
-    if (ctxt->controlBufIndex > 0) {
-	memmove(&ctxt->controlBuf[0], &ctxt->controlBuf[ctxt->controlBufIndex],
-		ctxt->controlBufUsed - ctxt->controlBufIndex);
-	ctxt->controlBufUsed -= ctxt->controlBufIndex;
-	ctxt->controlBufIndex = 0;
-    }
-    size = FTP_BUF_SIZE - ctxt->controlBufUsed;
-    if (size == 0) {
-#ifdef DEBUG_FTP
-        printf("xmlNanoFTPGetMore : buffer full %d \n", ctxt->controlBufUsed);
-#endif
-	return(0);
-    }
-
-    /*
-     * Read the amount left on teh control connection
-     */
-    if ((len = recv(ctxt->controlFd, &ctxt->controlBuf[ctxt->controlBufIndex],
-		    size, 0)) < 0) {
-	close(ctxt->controlFd); ctxt->controlFd = -1;
-        ctxt->controlFd = -1;
-        return(-1);
-    }
-#ifdef DEBUG_FTP
-    printf("xmlNanoFTPGetMore : read %d [%d - %d]\n", len,
-	   ctxt->controlBufUsed, ctxt->controlBufUsed + len);
-#endif
-    ctxt->controlBufUsed += len;
-    ctxt->controlBuf[ctxt->controlBufUsed] = 0;
-
-    return(len);
-}
-
 /**
  * xmlNanoFTPReadResponse:
  * @ctx:  an FTP context
+ * @buf:  buffer to read in
+ * @size:  buffer length
  *
  * Read the response from the FTP server after a command.
  * Returns the code number
  */
 static int
-xmlNanoFTPReadResponse(void *ctx) {
+xmlNanoFTPReadResponse(void *ctx, char *buf, int size) {
     xmlNanoFTPCtxtPtr ctxt = (xmlNanoFTPCtxtPtr) ctx;
     char *ptr, *end;
     int len;
-    int res = -1, cur = -1;
+    int res = -1;
+
+    if (size <= 0) return(-1);
 
 get_more:
-    /*
-     * Assumes everything up to controlBuf[controlBufIndex] has been read
-     * and analyzed.
-     */
-    len = xmlNanoFTPGetMore(ctx);
-    if ((ctxt->controlBufUsed == 0) && (len == 0)) {
+    if ((len = recv(ctxt->controlFd, buf, size - 1, 0)) < 0) {
+	close(ctxt->controlFd); ctxt->controlFd = -1;
+        ctxt->controlFd = -1;
+        return(-1);
+    }
+    if (len == 0) {
         return(-1);
     }
-    ptr = &ctxt->controlBuf[ctxt->controlBufIndex];
-    end = &ctxt->controlBuf[ctxt->controlBufUsed];
 
+    end = &buf[len];
+    *end = 0;
 #ifdef DEBUG_FTP
-    printf("\n<<<\n%s\n--\n", ptr);
+    printf(buf);
 #endif
+    ptr = buf;
     while (ptr < end) {
-        cur = xmlNanoFTPParseResponse(ctxt, ptr, end - ptr);
-	if (cur > 0) {
-	    /*
-	     * Successfully scanned the control code, scratch
-	     * till the end of the line, but keep the index to be
-	     * able to analyze the result if needed.
-	     */
-	    res = cur;
-	    ptr += 3;
-	    ctxt->controlBufAnswer = ptr - ctxt->controlBuf;
-	    while ((ptr < end) && (*ptr != '\n')) ptr++;
-	    if (*ptr == '\n') ptr++;
-	    if (*ptr == '\r') ptr++;
-	    break;
+        res = xmlNanoFTPParseResponse(ctxt, ptr, end - ptr);
+	if (res > 0) break;
+	if (res == 0) {
+#ifdef DEBUG_FTP
+	    fprintf(stderr, "xmlNanoFTPReadResponse failed: %s\n", ptr);
+#endif
+	    return(-1);
 	}
 	while ((ptr < end) && (*ptr != '\n')) ptr++;
 	if (ptr >= end) {
-	    ctxt->controlBufIndex = ctxt->controlBufUsed;
-	    goto get_more;
+#ifdef DEBUG_FTP
+	    fprintf(stderr, "xmlNanoFTPReadResponse: unexpected end %s\n", buf);
+#endif
+	    return((-res) / 100);
 	}
 	if (*ptr != '\r') ptr++;
     }
 
     if (res < 0) goto get_more;
-    ctxt->controlBufIndex = ptr - ctxt->controlBuf;
-#ifdef DEBUG_FTP
-    ptr = &ctxt->controlBuf[ctxt->controlBufIndex];
-    printf("\n---\n%s\n--\n", ptr);
-#endif
 
 #ifdef DEBUG_FTP
     printf("Got %d\n", res);
@@ -669,11 +588,22 @@ get_more:
 
 int
 xmlNanoFTPGetResponse(void *ctx) {
+    char buf[16 * 1024 + 1];
+
+/**************
+    fd_set rfd;
+    struct timeval tv;
     int res;
 
-    res = xmlNanoFTPReadResponse(ctx);
+    tv.tv_sec = 10;
+    tv.tv_usec = 0;
+    FD_ZERO(&rfd);
+    FD_SET(ctxt->controlFd, &rfd);
+    res = select(ctxt->controlFd + 1, &rfd, NULL, NULL, &tv);
+    if (res <= 0) return(res);
+ **************/
 
-    return(res);
+    return(xmlNanoFTPReadResponse(ctx, buf, 16 * 1024));
 }
 
 /**
@@ -687,6 +617,7 @@ xmlNanoFTPGetResponse(void *ctx) {
 int
 xmlNanoFTPCheckResponse(void *ctx) {
     xmlNanoFTPCtxtPtr ctxt = (xmlNanoFTPCtxtPtr) ctx;
+    char buf[1024 + 1];
     fd_set rfd;
     struct timeval tv;
 
@@ -705,7 +636,7 @@ xmlNanoFTPCheckResponse(void *ctx) {
 			
     }
 
-    return(xmlNanoFTPReadResponse(ctx));
+    return(xmlNanoFTPReadResponse(ctx, buf, 1024));
 }
 
 /**
@@ -830,11 +761,6 @@ xmlNanoFTPConnect(void *ctx) {
     if (hp == NULL)
         return(-1);
 
-    if ((unsigned int) hp->h_length >
-	sizeof(((struct sockaddr_in *)&ctxt->ftpAddr)->sin_addr)) {
-	return (-1);
-    }
-
     /*
      * Prepare the socket
      */
@@ -1138,14 +1064,15 @@ xmlNanoFTPConnect(void *ctx) {
 /**
  * xmlNanoFTPConnectTo:
  * @server:  an FTP server name
- * @port:  the port (use 21 if 0)
+ * @directory:  the port (use 21 if 0)
  *
  * Tries to open a control connection to the given server/port
  *
- * Returns an fTP context or NULL if it failed
+ * Returns and fTP context or NULL if it failed
  */
 
-void*
+
+void *
 xmlNanoFTPConnectTo(const char *server, int port) {
     xmlNanoFTPCtxtPtr ctxt;
     int res;
@@ -1153,7 +1080,7 @@ xmlNanoFTPConnectTo(const char *server, int port) {
     xmlNanoFTPInit();
     if (server == NULL) 
 	return(NULL);
-    ctxt = (xmlNanoFTPCtxtPtr) xmlNanoFTPNewCtxt(NULL);
+    ctxt = xmlNanoFTPNewCtxt(NULL);
     ctxt->hostname = xmlMemStrdup(server);
     if (port != 0)
 	ctxt->port = port;
@@ -1166,7 +1093,7 @@ xmlNanoFTPConnectTo(const char *server, int port) {
 }
 
 /**
- * xmlNanoFTPCwd:
+ * xmlNanoFTPGetConnection:
  * @ctx:  an FTP context
  * @directory:  a directory on the server
  *
@@ -1253,7 +1180,7 @@ xmlNanoFTPGetConnection(void *ctx) {
 	    close(ctxt->dataFd); ctxt->dataFd = -1;
 	    return(res);
 	}
-        res = xmlNanoFTPReadResponse(ctx);
+        res = xmlNanoFTPReadResponse(ctx, buf, sizeof(buf) -1);
 	if (res != 2) {
 	    if (res == 5) {
 	        close(ctxt->dataFd); ctxt->dataFd = -1;
@@ -1266,14 +1193,12 @@ xmlNanoFTPGetConnection(void *ctx) {
 	        ctxt->passive = 0;
 	    }
 	}
-	cur = &ctxt->controlBuf[ctxt->controlBufAnswer]; 
+	cur = &buf[4];
 	while (((*cur < '0') || (*cur > '9')) && *cur != '\0') cur++;
 	if (sscanf(cur, "%d,%d,%d,%d,%d,%d", &temp[0], &temp[1], &temp[2],
 	            &temp[3], &temp[4], &temp[5]) != 6) {
 	    fprintf(stderr, "Invalid answer to PASV\n");
-	    if (ctxt->dataFd != -1) {
-		close(ctxt->dataFd); ctxt->dataFd = -1;
-	    }
+	    close(ctxt->dataFd); ctxt->dataFd = -1;
 	    return(-1);
 	}
 	for (i=0; i<6; i++) ad[i] = (unsigned char) (temp[i] & 0xff);
@@ -1528,8 +1453,6 @@ xmlNanoFTPList(void *ctx, ftpListCallback callback, void *userData,
         if (xmlNanoFTPCwd(ctxt, ctxt->path) < 1)
 	    return(-1);
 	ctxt->dataFd = xmlNanoFTPGetConnection(ctxt);
-	if (ctxt->dataFd == -1)
-	    return(-1);
 #ifndef HAVE_SNPRINTF
 	len = sprintf(buf, "LIST -L\r\n");
 #else /* HAVE_SNPRINTF */
@@ -1541,8 +1464,6 @@ xmlNanoFTPList(void *ctx, ftpListCallback callback, void *userData,
 		return(-1);
 	}
 	ctxt->dataFd = xmlNanoFTPGetConnection(ctxt);
-	if (ctxt->dataFd == -1)
-	    return(-1);
 #ifndef HAVE_SNPRINTF
 	len = sprintf(buf, "LIST -L %s\r\n", filename);
 #else /* HAVE_SNPRINTF */
@@ -1557,7 +1478,7 @@ xmlNanoFTPList(void *ctx, ftpListCallback callback, void *userData,
 	close(ctxt->dataFd); ctxt->dataFd = -1;
 	return(res);
     }
-    res = xmlNanoFTPReadResponse(ctxt);
+    res = xmlNanoFTPReadResponse(ctxt, buf, sizeof(buf) -1);
     if (res != 1) {
 	close(ctxt->dataFd); ctxt->dataFd = -1;
 	return(-res);
@@ -1638,8 +1559,6 @@ xmlNanoFTPGetSocket(void *ctx, const char *filename) {
     if ((filename == NULL) && (ctxt->path == NULL))
 	return(-1);
     ctxt->dataFd = xmlNanoFTPGetConnection(ctxt);
-    if (ctxt->dataFd == -1)
-	return(-1);
 
 #ifndef HAVE_SNPRINTF
     len = sprintf(buf, "TYPE I\r\n");
@@ -1654,7 +1573,7 @@ xmlNanoFTPGetSocket(void *ctx, const char *filename) {
 	close(ctxt->dataFd); ctxt->dataFd = -1;
 	return(res);
     }
-    res = xmlNanoFTPReadResponse(ctxt);
+    res = xmlNanoFTPReadResponse(ctxt, buf, sizeof(buf) -1);
     if (res != 2) {
 	close(ctxt->dataFd); ctxt->dataFd = -1;
 	return(-res);
@@ -1679,7 +1598,7 @@ xmlNanoFTPGetSocket(void *ctx, const char *filename) {
 	close(ctxt->dataFd); ctxt->dataFd = -1;
 	return(res);
     }
-    res = xmlNanoFTPReadResponse(ctxt);
+    res = xmlNanoFTPReadResponse(ctxt, buf, sizeof(buf) -1);
     if (res != 1) {
 	close(ctxt->dataFd); ctxt->dataFd = -1;
 	return(-res);
@@ -1794,7 +1713,7 @@ xmlNanoFTPRead(void *ctx, void *dest, int len) {
  * Returns an FTP context, or NULL 
  */
 
-void*
+void *
 xmlNanoFTPOpen(const char *URL) {
     xmlNanoFTPCtxtPtr ctxt;
     int sock;
@@ -1803,7 +1722,7 @@ xmlNanoFTPOpen(const char *URL) {
     if (URL == NULL) return(NULL);
     if (strncmp("ftp://", URL, 6)) return(NULL);
 
-    ctxt = (xmlNanoFTPCtxtPtr) xmlNanoFTPNewCtxt(URL);
+    ctxt = xmlNanoFTPNewCtxt(URL);
     if (ctxt == NULL) return(NULL);
     if (xmlNanoFTPConnect(ctxt) < 0) {
 	xmlNanoFTPFreeCtxt(ctxt);
@@ -1898,3 +1817,12 @@ int main(int argc, char **argv) {
     exit(0);
 }
 #endif /* STANDALONE */
+#else /* !LIBXML_FTP_ENABLED */
+#ifdef STANDALONE
+#include <stdio.h>
+int main(int argc, char **argv) {
+    printf("%s : FTP support not compiled in\n", argv[0]);
+    return(0);
+}
+#endif /* STANDALONE */
+#endif /* LIBXML_FTP_ENABLED */
