@@ -21,7 +21,9 @@
 #include "config.h"
 #endif
 
+#include "xmlversion.h"
 
+#ifdef LIBXML_HTTP_ENABLED
 #include <stdio.h>
 #include <string.h>
 
@@ -59,8 +61,8 @@
 #include <strings.h>
 #endif
 
-#include "xmlmemory.h"
-#include "nanohttp.h"
+#include <libxml/xmlmemory.h>
+#include <libxml/nanohttp.h>
 
 #ifdef STANDALONE
 #define DEBUG_HTTP
@@ -161,7 +163,6 @@ xmlNanoHTTPScanURL(xmlNanoHTTPCtxtPtr ctxt, const char *URL) {
     const char *cur = URL;
     char buf[4096];
     int index = 0;
-    const int indexMax = 4096 - 1;
     int port = 0;
 
     if (ctxt->protocol != NULL) { 
@@ -178,7 +179,7 @@ xmlNanoHTTPScanURL(xmlNanoHTTPCtxtPtr ctxt, const char *URL) {
     }
     if (URL == NULL) return;
     buf[index] = 0;
-    while ((*cur != 0) && (index < indexMax)) {
+    while (*cur != 0) {
         if ((cur[0] == ':') && (cur[1] == '/') && (cur[2] == '/')) {
 	    buf[index] = 0;
 	    ctxt->protocol = xmlMemStrdup(buf);
@@ -220,7 +221,7 @@ xmlNanoHTTPScanURL(xmlNanoHTTPCtxtPtr ctxt, const char *URL) {
     else {
         index = 0;
         buf[index] = 0;
-	while ((*cur != 0) && (index < indexMax))
+	while (*cur != 0)
 	    buf[index++] = *cur++;
 	buf[index] = 0;
 	ctxt->path = xmlMemStrdup(buf);
@@ -242,7 +243,6 @@ xmlNanoHTTPScanProxy(const char *URL) {
     const char *cur = URL;
     char buf[4096];
     int index = 0;
-    const int indexMax = 4096 - 1;
     int port = 0;
 
     if (proxy != NULL) { 
@@ -260,7 +260,7 @@ xmlNanoHTTPScanProxy(const char *URL) {
 #endif
     if (URL == NULL) return;
     buf[index] = 0;
-    while ((*cur != 0) && (index < indexMax)) {
+    while (*cur != 0) {
         if ((cur[0] == ':') && (cur[1] == '/') && (cur[2] == '/')) {
 	    buf[index] = 0;
 	    index = 0;
@@ -755,7 +755,7 @@ retry:
     }
     ctxt->fd = ret;
     if (proxy) {
-#ifdef have_snprintf
+#ifdef HAVE_SNPRINTF
 	if (ctxt->port != 80)
 	    snprintf(buf, sizeof(buf),
 		     "GET http://%s:%d%s HTTP/1.0\r\nHost: %s\r\n\r\n",
@@ -1233,3 +1233,12 @@ int main(int argc, char **argv) {
     return(0);
 }
 #endif /* STANDALONE */
+#else /* !LIBXML_HTTP_ENABLED */
+#ifdef STANDALONE
+#include <stdio.h>
+int main(int argc, char **argv) {
+    printf("%s : HTTP support not compiled in\n", argv[0]);
+    return(0);
+}
+#endif /* STANDALONE */
+#endif /* LIBXML_HTTP_ENABLED */
