@@ -9,10 +9,10 @@
 #ifndef __XML_PARSER_H__
 #define __XML_PARSER_H__
 
-#include "tree.h"
-#include "valid.h"
-#include "xmlIO.h"
-#include "entities.h"
+#include <libxml/tree.h>
+#include <libxml/valid.h>
+#include <libxml/xmlIO.h>
+#include <libxml/entities.h>
 
 
 #ifdef __cplusplus
@@ -49,6 +49,9 @@ struct _xmlParserInput {
     int col;                          /* Current column */
     int consumed;                     /* How many xmlChars already consumed */
     xmlParserInputDeallocate free;    /* function to deallocate the base */
+    const xmlChar *encoding;          /* the encoding string for entity */
+    const xmlChar *version;           /* the version string for entity */
+    int standalone;                   /* Was that entity marked standalone */
 };
 
 /**
@@ -95,6 +98,7 @@ typedef enum {
     XML_PARSER_ENTITY_DECL,	/* within an entity declaration */
     XML_PARSER_ENTITY_VALUE,	/* within an entity value in a decl */
     XML_PARSER_ATTRIBUTE_VALUE,	/* within an attribute value */
+    XML_PARSER_SYSTEM_LITERAL,	/* within a SYSTEM value */
     XML_PARSER_EPILOG 		/* the Misc* after the last end tag */
 } xmlParserInputState;
 
@@ -151,7 +155,7 @@ struct _xmlParserCtxt {
 
     char           *directory;        /* the data directory */
 
-    /* Node name stack only used for HTML parsing */
+    /* Node name stack */
     xmlChar           *name;          /* Current parsed Node */
     int                nameNr;        /* Depth of the parsing stack */
     int                nameMax;       /* Max depth of the parsing stack */
@@ -160,6 +164,20 @@ struct _xmlParserCtxt {
     long               nbChars;       /* number of xmlChar processed */
     long            checkIndex;       /* used by progressive parsing lookup */
     int             keepBlanks;       /* ugly but ... */
+    int             disableSAX;       /* SAX callbacks are disabled */
+    int               inSubset;       /* Parsing is in int 1/ext 2 subset */
+    xmlChar *          intSubName;    /* name of subset */
+    xmlChar *          extSubURI;     /* URI of external subset */
+    xmlChar *          extSubSystem;  /* SYSTEM ID of external subset */
+
+    /* xml:space values */
+    int *              space;         /* Should the parser preserve spaces */
+    int                spaceNr;       /* Depth of the parsing stack */
+    int                spaceMax;      /* Max depth of the parsing stack */
+    int *              spaceTab;      /* array of space infos */
+
+    int                depth;         /* to prevent entity substitution loops */
+    xmlParserInputPtr  entity;      /* used to check entities boundaries */
 };
 
 /**
@@ -183,6 +201,8 @@ typedef xmlParserInputPtr (*resolveEntitySAXFunc) (void *ctx,
 			    const xmlChar *publicId, const xmlChar *systemId);
 typedef void (*internalSubsetSAXFunc) (void *ctx, const xmlChar *name,
                             const xmlChar *ExternalID, const xmlChar *SystemID);
+typedef void (*externalSubsetSAXFunc) (void *ctx, const xmlChar *name,
+                            const xmlChar *ExternalID, const xmlChar *SystemID);
 typedef xmlEntityPtr (*getEntitySAXFunc) (void *ctx,
                             const xmlChar *name);
 typedef xmlEntityPtr (*getParameterEntitySAXFunc) (void *ctx,
@@ -254,6 +274,7 @@ struct _xmlSAXHandler {
     fatalErrorSAXFunc fatalError;
     getParameterEntitySAXFunc getParameterEntity;
     cdataBlockSAXFunc cdataBlock;
+    externalSubsetSAXFunc externalSubset;
 };
 
 /**
@@ -278,7 +299,7 @@ extern xmlSAXHandler htmlDefaultSAXHandler;
  */
 
 extern int xmlSubstituteEntitiesDefaultValue;
-
+extern int xmlGetWarningsDefaultValue;
 
 
 /**
@@ -363,6 +384,20 @@ xmlDtdPtr	xmlParseDTD		(const xmlChar *ExternalID,
 xmlDtdPtr	xmlSAXParseDTD		(xmlSAXHandlerPtr sax,
 					 const xmlChar *ExternalID,
 					 const xmlChar *SystemID);
+int		xmlParseBalancedChunkMemory(xmlDocPtr doc,
+					 xmlSAXHandlerPtr sax,
+					 void *user_data,
+					 int depth,
+					 const xmlChar *string,
+					 xmlNodePtr *list);
+int		xmlParseExternalEntity	(xmlDocPtr doc,
+					 xmlSAXHandlerPtr sax,
+					 void *user_data,
+					 int depth,
+					 const xmlChar *URL,
+					 const xmlChar *ID,
+					 xmlNodePtr *list);
+
 /**
  * SAX initialization routines
  */
@@ -393,6 +428,21 @@ int		 xmlParseChunk		(xmlParserCtxtPtr ctxt,
 					 int size,
 					 int terminate);
 
+/**
+ * Special I/O mode
+ */
+
+xmlParserCtxtPtr xmlCreateIOParserCtxt	(xmlSAXHandlerPtr sax,
+					 void *user_data,
+					 xmlInputReadCallback   ioread,
+					 xmlInputCloseCallback  ioclose,
+					 void *ioctx,
+					 xmlCharEncoding enc);
+
+xmlParserInputPtr xmlNewIOInputStream	(xmlParserCtxtPtr ctxt,
+					 xmlParserInputBufferPtr input,
+					 xmlCharEncoding enc);
+
 /**
  * Node infos
  */
