@@ -43,8 +43,6 @@ xmlURIPtr	xmlCreateURI		(void);
 xmlChar *	xmlBuildURI		(const xmlChar *URI,
 	                        	 const xmlChar *base);
 xmlURIPtr	xmlParseURI		(const char *URI);
-int		xmlParseURIReference	(xmlURIPtr uri,
-					 const char *str);
 xmlChar *	xmlSaveUri		(xmlURIPtr uri);
 void		xmlPrintURI		(FILE *stream,
 					 xmlURIPtr uri);
@@ -52,7 +50,6 @@ char *		xmlURIUnescapeString	(const char *str,
 					 int len,
 					 char *target);
 int		xmlNormalizeURIPath	(char *path);
-xmlChar *	xmlURIEscape		(const xmlChar *str);
 void		xmlFreeURI		(xmlURIPtr uri);
 
 #ifdef __cplusplus
