@@ -1,7 +1,7 @@
 #ifndef __XML_ERROR_H__
 #define __XML_ERROR_H__
 
-#include "parser.h"
+#include <libxml/parser.h>
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -117,7 +117,6 @@ typedef enum {
 
     XML_ERR_HYPHEN_IN_COMMENT, /* 81 */
 
-    /* Added after 2.3.5 integration */
     XML_ERR_INVALID_ENCODING, /* 82 */
 
     XML_ERR_EXT_ENTITY_STANDALONE, /* 83 */
@@ -131,9 +130,7 @@ typedef enum {
     XML_ERR_ENTITY_CHAR_ERROR, /* 88 */
     XML_ERR_ENTITY_PE_INTERNAL, /* 88 */
     XML_ERR_ENTITY_LOOP, /* 89 */
-    XML_ERR_ENTITY_BOUNDARY, /* 90 */
-    XML_ERR_INVALID_URI, /* 91 */
-    XML_ERR_URI_FRAGMENT /* 92 */
+    XML_ERR_ENTITY_BOUNDARY /* 90 */
 }xmlParserErrors;
 
 void	xmlParserError		(void *ctx,
